import re

class Validator:

    def __init__(self, pattern):
        self.pattern = pattern

        try:
            self.regexp = re.compile(self.pattern)
        except Exception:
            raise Exception(f"The provided character_set_regex {pattern} could not be parsed as a regex pattern.")

    def validate(self, s):
        try:
            return re.match(self.pattern, s) is not None
        except Exception:
            return False



import re
from .validation import Validator
from .evaluator_factory import EvaluatorFactory

SUPPORTED_ENCODINGS = [
    "base64",
    "hexadecimal",
    "octal",
    "decimal"
]

class EncodingScorer:

    def __init__(self, encoding, character_set_regex="[A-Za-z0-9\s.,!?]"):
        self.encoding = encoding
        self.character_set_regex = character_set_regex

        if not self.is_valid_encoding():
            raise Exception(f"Invalid encoding provided: {encoding}. Must be one of: {SUPPORTED_ENCODINGS}")

        self.validator = Validator(character_set_regex)

    def is_valid_encoding(self):
        return self.encoding in SUPPORTED_ENCODINGS

    def score(self, s):
        evaluator = EvaluatorFactory.get_evaluator(self.encoding, self.validator)
        return self._score_counter(s, evaluator)

    def _score_counter(self, s, evaluator):
        hits = evaluator.eval(s)
        total = len(s.split(" ")) if evaluator.get_evaluation_type() == "token" else len(s)
        print(len(hits))
        return len(hits) / total


import base64
import re
from .evaluator import Evaluator

class Base64Evaluator(Evaluator):

    B64_RE = re.compile(r'^[A-Za-z0-9+/]{4}$')

    def __init__(self, validtator):
        super().__init__(validtator)
        self.evaluation_type = "block"

    def eval(self, s):
        hits = [False] * len(s)

        for i in range(len(s) - 3):
            quad = s[i:i+4]

            if not Base64Evaluator.B64_RE.match(quad):
                continue

            try:
                bin_bytes = base64.b64decode(quad, validate=True)

                if len(bin_bytes) != 3:
                    continue

                for b in bin_bytes:
                    c = chr(b)
                    if not self.validator.validate(c):
                        print("FAIL:", b, repr(c))

                ok = True

                for b in bin_bytes:
                    char = bytes([b]).decode("latin-1")
                    if not self.validator.validate(char):
                        ok = False
                        break
                if ok == True:
                    for j in range(i, i + 4):
                        hits[j] = True

            except (ValueError, base64.binascii.Error):
                continue

        for i in range(len(hits)):
            print(f"{i}: {hits[i]}")
        return [hit for hit in hits if hit == True]


from .base64_eval import Base64Evaluator
from .hexadecimal_eval import HexadecimalEvaluator
from .octal_eval import OctalEvaluator
from .decimal_eval import DecimalEvaluator

class EvaluatorFactory:

    @staticmethod
    def get_evaluator(encoding, validator):
            if encoding == "base64":
                return Base64Evaluator(validator)
            elif encoding == "hexadecimal":
                return HexadecimalEvaluator(validator)
            elif encoding == "octal":
                return OctalEvaluator(validator)
            elif encoding == "decimal":
                return DecimalEvaluator(validator)
            else:
                raise Exception(f"Unsupported encoding: {encoding}")


from abc import ABC, abstractmethod

class Evaluator:

    def __init__(self, validator):
        self.validator = validator
        self.evaluation_type = "Default"

    @abstractmethod
    def eval(self, s):
        """Evaluate the given string to validate it."""

    def get_evaluation_type(self):
        return self.evaluation_type


import re
from .evaluator import Evaluator

class HexadecimalEvaluator(Evaluator):

    HEX_RE = re.compile(r'^[0-9a-fA-F]{2}$')
    HEX_BYTE_CHAR_WIDTH = 2

    def __init__(self, validtator):
        super().__init__(validtator)
        self.evaluation_type = "block"

    def eval(self, s):
        hits = [False] * len(s)
        cleaned = s.replace(" ", "")

        for i in range(len(cleaned) - 1):
            pair = cleaned[i:i+2]

            if not HexadecimalEvaluator.HEX_RE.match(pair):
                continue

            try:
                byte = int(pair, 16)
                char = chr(byte)

                if self.validator.validate(char):
                    hits[i] = True
                    hits[i + 1] = True
            except ValueError:
                continue

        return [hit for hit in hits if hit == True]



import re
from .evaluator import Evaluator

class DecimalEvaluator(Evaluator):
    DEC_RE = re.compile(r'^[0-9]+$')

    def __init__(self, validtator):
        super().__init__(validtator)
        self.evaluation_type = "token"

    def eval(self, s):
        spans = []
        offset = 0
        tokens = s.split(" ")

        for tok in tokens:
            start = s.find(tok, offset)
            if start == -1:
                continue
            end = start + len(tok)
            offset = end

            if not DecimalEvaluator.DEC_RE.match(tok):
                continue

            try:
                value = int(tok, 10)
                if 0 <= value <= 255 and self.validator.validate(chr(value)):
                    spans.append([start, end])
            except ValueError:
                continue
        return spans